import java.util.*;
import java.io.*;

/**
 * Prototype - Engine class.
 * 
 * Where the bulk of things are dealt with.
 * 
 * @author Team NAH
 * @version Spring 2006
 */
public class Engine {
	// references to other classes in the project

	private GUI gui;

	// //////////////BEGIN: SINGLETON CODE////////////////////

	// unique static reference to the engine
	private static Engine ref;

	private Engine() {
		// method is never called
	}

	/**
	 * Get a unique instance of the engine object.
	 * 
	 * Returns the unique instance of engine (creates it if it doesn't already
	 * exist.)
	 */
	public static Engine getEngine() {
		if (ref == null) {
			ref = new Engine();
		}
		return ref;
	}

	/**
	 * Intercepts any attempts to clone the object.
	 * 
	 * This method exploits polymorphism by intercepting attempts to clone the
	 * object and throwing an exception. By doing this, the class is guaranteed
	 * not to be duplicated.
	 */
	public Object clone() throws CloneNotSupportedException {
		throw new CloneNotSupportedException();
	}

	// //////////////END: SINGLETON CODE////////////////////

	// //////////////BEGIN: UTILITY CODE////////////////////

	/**
	 * Perform initial operations.
	 * 
	 * Calls the GUI and assembles it. Will also load files etc.
	 */
	public void initialise() {
		initilise_error_handler();
		SplashScreen splashScreen = new SplashScreen("Splash.png");
		splashScreen.splash();
		IsotopeLibrary.getLibrary().initilise("ISOTOPES");
		try {
			Thread.sleep(2000);
		} catch (InterruptedException e) {
			System.out.println(e);
		}
		splashScreen.dispose();

		gui = new GUI();
		gui.assemble();
	}

	/**
	 * <p>
	 * Initilise the error handler
	 * </p>
	 */
	private void initilise_error_handler() {
		ErrorSubject.getErrorSubject()
				.attachObserver(new DefaultErrorHandler());
	}

	// /////////////END: UTILITY CODE///////////////////////

	public void formulaOkBtnPressed(String str) {
		/*StringReader reader = new StringReader(str);
		ChemParser p = new ChemParser(reader);
		ASTInput chem;*/
		
		////
		Formula_Parser parser = new Formula_Parser(str);
		////
		
		/*
		try {
			chem = p.Input();
			chem.dump("");

		} catch (Exception e) {
			gui.setErrLabel("Parse Error");
		}*/
		
		// example!
		/*
		String xlabels[] = { "", "191", "192", "193", "194", "195", "196",
				"197", "198", "199", "200" };
		String ylabels[] = { "", "10 ", "20 ", "30 ", "40 ", "50 ", "60 ",
				"70 ", "80 ", "90 ", "100" };
		String origin = "0";

		GraphDataSet dataSet = new GraphDataSet(xlabels, ylabels, origin, null);

		Point aPoint = new Point(5, 100);
		Point bPoint = new Point(6, 10);
		Point cPoint = new Point(7, 1);
		Point dPoint = new Point(8, 0);
		dataSet.addPoint(aPoint);
		dataSet.addPoint(bPoint);
		dataSet.addPoint(cPoint);
		dataSet.addPoint(dPoint);
		*/
		
		//end example
		
		gui.setData( parser.getGraphDataSet()/*dataSet*/);
	}

	/**
	 * Main method.
	 * 
	 * Instantiates the engine and calls the initialise method.
	 */
	public static void main(String args[]) {
		Engine engine = getEngine();
		engine.initialise();
	}
}
