import java.util.Vector;
import java.util.Enumeration;

/**
 * Error Subject
 * 
 * This class manages all the error handlers
 * 
 * All errors should be sent to <code>quickError</code>
 * 
 * I will produce some uml diagrams and put in the documentation folder.
 * 
 * @author Team NAH
 * @version Spring 2006
 */
public class ErrorSubject {

	// //////////////BEGIN: SINGLETON CODE////////////////////

	// unique static reference to the engine
	private static ErrorSubject ref;

	private ErrorSubject() {
		vHandlers = new Vector(2);
	}

	/**
	 * Get a unique instance of the engine object.
	 * 
	 * Returns the unique instance of engine (creates it if it doesn't already
	 * exist.)
	 */
	public static ErrorSubject getErrorSubject() {
		if (ref == null) {
			ref = new ErrorSubject();
		}
		return ref;
	}

	/**
	 * Intercepts any attempts to clone the object.
	 * 
	 * This method exploits polymorphism by intercepting attempts to clone the
	 * object and throwing an exception. By doing this, the class is guaranteed
	 * not to be duplicated.
	 */
	public Object clone() throws CloneNotSupportedException {
		throw new CloneNotSupportedException();
	}

	// //////////////END: SINGLETON CODE////////////////////

	// ////////////BEGIN: STATIC OPERATIONS/////////////////
	/**
	 * Starts the processing of an error message, short cut
	 */
	public static void quickError(int errorID, String msg) {
		getErrorSubject().processError(errorID, msg);
	}

	// //////////////END: STATIC OPERATIONS/////////////////

	// ////////////BEGIN: INSTANCE CODE/////////////////////
	protected Vector vHandlers;

	/**
	 * Starts the processing of an error message
	 */
	public void processError(int errorID, String msg) {
		Enumeration enumeration = vHandlers.elements();

		while (enumeration.hasMoreElements()) {
			((IErrorObserver) enumeration.nextElement()).handleError(errorID,
					msg);
		}
	}

	/**
	 * Attaches an error handler to the hash table of error handlers.
	 */
	public void attachObserver(IErrorObserver obj) {
		vHandlers.addElement(obj);
	}

	/**
	 * Removes an error handler from the hash table of error handlers.
	 */
	public void removeObserver(IErrorObserver obj) {
		vHandlers.remove(obj);
	}
	// //////////////END: INSTANCE CODE/////////////////////

}
